function score=IF_wrap(train,test,param,aggregation,score_types,nTrees)
% Enhanced Anomaly Scores for Isolation Forest, v1.0, 2024
% (c) A. Mensi
%
% score = IF_wrap(train,test,param,score_types) builds a IF on the train matrix
% and provides one or more (anomaly) scores for each object in the test matrix.
%
% train is a  matrix Nxd
%
% test is a matrix Mxd
%
% param is a structure containing several fields which refer to the
% parameters of the IF model. For more information write 'help
% 'IF_training'
%
% aggregation is a value indicating how tree scores are combined at forest
% level. The possible values are 'sx' or 'px'.
%
% score_types is a vector that can contain one or more values in the range
% of 0:5 indicating a specific path-weighted variant. 0 corresponds to no
% weight, whereas the remainders correspond to the V1-V5 variants indicated
% in the paper "Enhanced Anomaly scores for Isolation Forests," Pattern
% Recognit., 2021, A. Mensi, M. Bicego
%
% score is a vector/matrix of anomaly scores, of dimensions Mxs where s is the
% length of score_types. Higher values indicate higher anomalousness.
forest=IF_training(train,param);
c_l=zeros(forest.param.S,1);
for i=1:forest.param.S
    c_l(i)=average_unsuccessful_search(i);
end
if nargin <6
    nTrees = forest.param.T;
else
    nTrees = min(nTrees,forest.param.T);
end
score=IF_enhancedTesting(forest,test,aggregation,score_types,nTrees,c_l);
end