function param = setIFParameters(param,N,d)
% Enhanced Anomaly Scores for Isolation Forest, v1.0, 2024
% (c) A. Mensi
%
% param = setIFParameters(param,N) checks and sets the parameters of
% the IF model to be built.
% param is the structure of parameters
% N is the number of training objects
% d is the number of features

if nargin<2
    error("You must pass the number of objects in the training set.\n")
end
if ~isfield(param,'T') || param.T<1
    param.T=100;
elseif ~isequal(class(param.T),'double')
    fprintf("T must be a number! Setting it to default value T=100\n");
    param.T=100;
end

if ~isfield(param,'S')
    param.S=min(256,N);
    fprintf("S is not set! Setting it to default value S=min(256,N)\n")
elseif ~isequal(class(param.S),'double')
    fprintf("S must be a number!Setting it to default value S=min(256,N)\n");
    param.S=min(256,N);
end

if param.S>N
    fprintf("S is bigger than your number of samples N! Setting S=N!\n");
    param.S=N;
end

if ~isfield(param,'D')
    fprintf("D is not set! Setting it to default value D='log'\n");
    param.D='log';
elseif ~any(strcmpi(param.D,{'log','max'}))
    fprintf("D must either be 'log' or 'max'! Setting it to default value D='log'\n");
    param.D='log';
end
if strcmpi(param.D,'log')
    param.max_depth=ceil(log2(param.S));
else %param.D is 'max'
    param.max_depth=param.S-1;
end

if ~isfield(param,'F') %Features Subsampled to build each tree, expressed in percentage
    fprintf("F is not set, setting it to default value F=1\n");
    param.F = 1;
elseif ~isequal(class(param.F),'double')
    fprintf("F must be a number, setting it to default value F=1\n");
    param.F = 1;
elseif param.F<=0 || param.F>1
    fprintf("F must be a value  in (0,1] , setting it to default value F=1\n");
    param.F = 1;
end
if param.F==1
    param.nFeat=d;
else
    param.nFeat=ceil(param.F*d);
end

if ~isfield(param,'gamma') || param.gamma<=0 || param.gamma>1 %Used in the computation of the proxy (V2 and V3)
    fprintf("gamma must  be a value in (0,1], setting it to default value gamma=1\n");
    param.gamma=1;
end
end