%%% EXAMPLE OF HOW TO USE THE EnhancedAnomalyScores package %%%
clear all;close all;clc

%Number of iterations of the procedure
nIt=10;

%Inlier and outlier class labels
inl=0; outl=1;

%Adding the path
addpath(genpath('./EnhancedAnomalyScores_Code/'));

%Loading the dataset Annthyroid, downloaded from UCI. The dataset has already been processed: 1) the original training and test partitions have been merged, and
% 2) it has partitioned into inlier and outlier class (the latter formed by the two abnormal classes originally present)
load('annthyroid.mat')

%Finding the number and indexes of outliers and inliers in the dataset
N=size(data,1); NOutl=sum(labels==outl); NInl=N-NOutl;
inlIdx=find(labels==inl); outlIdx=find(labels==outl);
%data = (data-mean(data))./std(data); Uncomment if you want to normalize prior to train/test split + comment lines 51-52

%Setting the number and type of objects within the training partitions
Ntr=ceil(N.*0.5);
NtrO=0; %Training is without outliers
NtrI=Ntr-NtrO;

%Setting the forest parameters.
%You can also not set the fields of param, parameters will be set in the
%training function according to a fixed parametrization.
param.T=100;
param.D='log'; param.gamma=1;
param.F=1; param.S=min(256,Ntr);

%Setting testing parameters.
score_types = 0:5; %it indicates which path-weighted variants to compute. 0 corresponds to no weight, whereas the remainders correspond to the V1-V5 variants indicated in the paper
aggregation = 'px'; %It indicates how scores are aggregated at forest level. px is the novel variant defined within the paper. Alternative (and default) is sx.

auc=zeros(nIt,length(score_types));
fprintf("Iteration..")
for it=1:nIt
    fprintf("%d..\n",it);

    %Partitioning the data
    ordI=inlIdx(randperm(NInl));
    trainIdx=ordI(1:NtrI);
    testIdx=[ordI(NtrI+1:end); outlIdx];
    train=data(trainIdx,:); test=data(testIdx,:);
    labels_test=labels(testIdx);

    %Normalization according to train mean and std.
    mean_tr = mean(train); std_tr = std(train);
    train = (train-mean_tr)./std_tr; test = (test-mean_tr)./std_tr;

    %Training and testing procedure -- please write "help + function name" for details on how to use them
    score=IF_wrap(train,test,param,aggregation,score_types);
    %Alternative to the usage of the IF_wrap function:
    %forest=IF_training(train,param);
    %score=IF_enhancedTesting(forest,test,aggregation,score_types);
    for i = 1:size(score,1)
        [~,~,~,auc(it,i)]=perfcurve(labels_test,score(i,:),outl);
    end
end

fprintf("\nAnnthryoid, Table 4 AUC, p(x) unweighted: 0.927 VS found AUC %5.4f\n", mean(auc(:,1)));